#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 Olivier Tilloy <olivier@tilloy.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Scrape the data from bicing.com to get real-time information about the state of
the Bicing stations through the city of Barcelona.
"""

import re
import os
import urllib2
from datetime import datetime
from xml.dom import minidom
import sys
import pickle

# Where to get the data from
URL = 'http://bicing.com/localizaciones/localizaciones.php'

# How to scrape the data
XML_RE = re.compile(r"exml.parseString\('(.*)'\);")


class BicingStation(object):

    """
    A class representing a bicing station.

    @ivar number:      a unique integer (> 0) identifier for the station
    @type number:      C{int}
    @ivar name:        the full name of the station
    @type name:        C{unicode}
    @ivar coordinates: the GPS coordinates of the station
    @type coordinates: C{list} of C{float}
    @ivar bikes:       the number of bikes available in the station
    @type bikes:       C{int}
    @ivar free:        the number of free places in the station
    @type free:        C{int}
    @ivar timestamp:   when the state of the station was last updated
    @type timestamp:   L{datetime.datetime}
    """

    # How to pretty-print the timestamp
    _tsformat = '%Y-%m-%d %H:%M:%S'

    def __init__(self, number):
        """
        Constructor. Initialize all the fields.

        @param number: the identifier of the station
        @type number:  C{int}
        """
        self.number = number
        self.name = u''
        self.coordinates = []
        self.bikes = 0
        self.free = 0
        self.timestamp = datetime.now()

    def from_xml(self, placemark):
        """
        Set the values of the fields from an XML placemark node.

        @param placemark: the XML node containing the data about the station
        @type placemark:  L{xml.dom.minidom.Element}
        """
        description = placemark.getElementsByTagName('description')[0]
        xml = description.firstChild.nodeValue
        dom = minidom.parseString(xml.encode('utf-8'))
        divs = dom.firstChild.getElementsByTagName('div')
        self.name = divs[0].firstChild.nodeValue
        self.bikes = int(divs[2].childNodes[0].nodeValue)
        self.free = int(divs[2].childNodes[2].nodeValue)
        point = placemark.getElementsByTagName('Point')[0]
        coord = point.firstChild.firstChild.nodeValue
        self.coordinates = map(float, coord.split(','))

    def __repr__(self):
        """
        Return a human-readable text representation of the station.

        @return: a human-readable text representation of the station
        @rtype:  C{str}
        """
        result = '%d) %s' % (self.number, self.name.encode('utf-8'))
        result += ' (%s)' % ', '.join(map(str, self.coordinates)) + os.linesep
        result += '  last updated: %s' % self.timestamp.strftime(self._tsformat)
        result += os.linesep + '  bikes: %d' % self.bikes
        result += os.linesep + '  free: %d' % self.free
        return result


if __name__ == '__main__':
    page = urllib2.urlopen(URL)
    date = page.info()['Date']
    format = '%a, %d %b %Y %H:%M:%S %Z'
    try:
        timestamp = datetime.strptime(date, format)
    except AttributeError:
        # datetime.strptime is new in Python 2.5
        import time
        timestamp = datetime(*(time.strptime(date, format)[0:6]))
    content = page.read()
    page.close()
    xml = XML_RE.search(content).groups()[0]
    dom = minidom.parseString(xml.decode('iso-8859-15').encode('utf-8'))
    placemarks = dom.getElementsByTagName('Placemark')
    stations = []
    for index, placemark in enumerate(placemarks):
        station = BicingStation(index + 1)
        station.from_xml(placemark)
        station.timestamp = timestamp
        stations.append(station)
    outdir = sys.argv[1]
    if not os.path.exists(outdir):
        os.mkdir(outdir)
    outfile = timestamp.strftime('bicing_%Y%m%d_%H%M%S')
    outpath = os.path.join(outdir, outfile)
    fd = open(outpath, 'wb')
    pickle.dump(stations, fd)
    fd.close()
